/*
 *  Copyright (c) 2010 The VP8 project authors. All Rights Reserved.
 *
 *  Use of this source code is governed by a BSD-style license 
 *  that can be found in the LICENSE file in the root of the source
 *  tree. An additional intellectual property rights grant can be found
 *  in the file PATENTS.  All contributing project authors may 
 *  be found in the AUTHORS file in the root of the source tree.
 */


/*
 * This file is to be included at the top of the header files defining the
 * interface to individual codecs and contains various workarounds common
 * to all codec implementations.
 */
#ifdef __cplusplus
extern "C" {
#endif
